@@ -17,9 +17,7 @@
 #include "bson/bson.h"
 
 #include <php.h>
-#if PHP_VERSION_ID >= 80100
 #include <Zend/zend_enum.h>
-#endif
 #include <Zend/zend_interfaces.h>
 #include <Zend/zend_portability.h>
 
@@ -67,11 +65,9 @@ static inline bool phongo_is_class_instantiatable(const zend_class_entry* ce)
 		return false;
 	}
 
-#if PHP_VERSION_ID >= 80100
 	if (ce->ce_flags & ZEND_ACC_ENUM) {
 		return false;
 	}
-#endif /* PHP_VERSION_ID < 80100 */
 
 	return true;
 }
@@ -185,12 +181,15 @@ void php_phongo_field_path_write_type_at_current_level(php_phongo_field_path* fi
 
 bool php_phongo_field_path_push(php_phongo_field_path* field_path, const char* element, php_phongo_bson_field_path_item_types element_type)
 {
-	php_phongo_field_path_write_item_at_current_level(field_path, element);
+	if (element) {
+		php_phongo_field_path_write_item_at_current_level(field_path, element);
+	}
+
 	php_phongo_field_path_write_type_at_current_level(field_path, element_type);
 
 	field_path->size++;
 
-	return true;
+	return field_path->size <= BSON_MAX_NESTING_LEVEL;
 }
 
 bool php_phongo_field_path_pop(php_phongo_field_path* field_path)
@@ -776,7 +775,7 @@ static bool php_phongo_bson_visit_document(const bson_iter_t* iter ARG_UNUSED, c
 
 			object_init_ex(&obj, obj_ce);
 
-			zend_call_method_with_1_params(PHONGO_COMPAT_OBJ_P(&obj), NULL, NULL, BSON_UNSERIALIZE_FUNC_NAME, NULL, &state.zchild);
+			zend_call_method_with_1_params(Z_OBJ_P(&obj), NULL, NULL, BSON_UNSERIALIZE_FUNC_NAME, NULL, &state.zchild);
 			zval_ptr_dtor(&state.zchild);
 			ZVAL_COPY_VALUE(&state.zchild, &obj);
 
@@ -856,7 +855,7 @@ static bool php_phongo_bson_visit_array(const bson_iter_t* iter ARG_UNUSED, cons
 			zval obj;
 
 			object_init_ex(&obj, state.field_type.ce);
-			zend_call_method_with_1_params(PHONGO_COMPAT_OBJ_P(&obj), NULL, NULL, BSON_UNSERIALIZE_FUNC_NAME, NULL, &state.zchild);
+			zend_call_method_with_1_params(Z_OBJ_P(&obj), NULL, NULL, BSON_UNSERIALIZE_FUNC_NAME, NULL, &state.zchild);
 			zval_ptr_dtor(&state.zchild);
 			ZVAL_COPY_VALUE(&state.zchild, &obj);
 			break;
@@ -1147,7 +1146,7 @@ bool php_phongo_bson_to_zval_ex(const bson_t* b, php_phongo_bson_state* state)
 
 			object_init_ex(&obj, obj_ce);
 
-			zend_call_method_with_1_params(PHONGO_COMPAT_OBJ_P(&obj), NULL, NULL, BSON_UNSERIALIZE_FUNC_NAME, NULL, &state->zchild);
+			zend_call_method_with_1_params(Z_OBJ_P(&obj), NULL, NULL, BSON_UNSERIALIZE_FUNC_NAME, NULL, &state->zchild);
 			zval_ptr_dtor(&state->zchild);
 			ZVAL_COPY_VALUE(&state->zchild, &obj);
 
@@ -1450,7 +1449,7 @@ bool php_phongo_bson_to_json(zval* return_value, const bson_t* bson, php_phongo_
 	size_t json_len;
 
 	if (mode == PHONGO_JSON_MODE_LEGACY) {
-		json = bson_as_json(bson, &json_len);
+		json = bson_as_legacy_extended_json(bson, &json_len);
 	} else if (mode == PHONGO_JSON_MODE_CANONICAL) {
 		json = bson_as_canonical_extended_json(bson, &json_len);
 	} else if (mode == PHONGO_JSON_MODE_RELAXED) {
