@@ -17,9 +17,7 @@
 #include "bson/bson.h"
 
 #include <php.h>
-#if PHP_VERSION_ID >= 80100
 #include <Zend/zend_enum.h>
-#endif
 #include <Zend/zend_interfaces.h>
 
 #include "php_phongo.h"
@@ -58,6 +56,10 @@ static int php_phongo_is_array_or_document(zval* val)
 	int        count;
 
 	if (Z_TYPE_P(val) != IS_ARRAY) {
+		if (Z_TYPE_P(val) == IS_OBJECT && instanceof_function(Z_OBJCE_P(val), php_phongo_packedarray_ce)) {
+			return IS_ARRAY;
+		}
+
 		return IS_OBJECT;
 	}
 
@@ -86,6 +88,53 @@ static int php_phongo_is_array_or_document(zval* val)
 	return IS_ARRAY;
 }
 
+/* Checks the return type of a bsonSerialize() method. Returns true on
+ * success; otherwise, throws an exception and returns false.
+ *
+ * TODO: obsolete once the tentative return type in Serializable::bsonSerialize is enforced.
+ */
+static inline bool phongo_check_bson_serialize_return_type(zval* retval, zend_class_entry* ce)
+{
+	if (instanceof_function(ce, php_phongo_persistable_ce)) {
+		// Instances of Persistable must return an array, stdClass, or MongoDB\BSON\Document
+		if (
+			Z_TYPE_P(retval) != IS_ARRAY && !(Z_TYPE_P(retval) == IS_OBJECT && (instanceof_function(Z_OBJCE_P(retval), zend_standard_class_def) || instanceof_function(Z_OBJCE_P(retval), php_phongo_document_ce)))) {
+			phongo_throw_exception(PHONGO_ERROR_UNEXPECTED_VALUE,
+								   "Expected %s::%s() to return an array, stdClass, or %s, %s given",
+								   ZSTR_VAL(ce->name),
+								   BSON_SERIALIZE_FUNC_NAME,
+								   ZSTR_VAL(php_phongo_document_ce->name),
+								   zend_zval_type_name(retval));
+			return false;
+		}
+
+		return true;
+	}
+
+	if (instanceof_function(ce, php_phongo_serializable_ce)) {
+		// Instances of Serializable must return an array, stdClass, MongoDB\BSON\Document, or MongoDB\BSON\PackedArray
+		if (
+			Z_TYPE_P(retval) != IS_ARRAY && !(Z_TYPE_P(retval) == IS_OBJECT && (instanceof_function(Z_OBJCE_P(retval), zend_standard_class_def) || instanceof_function(Z_OBJCE_P(retval), php_phongo_document_ce) || instanceof_function(Z_OBJCE_P(retval), php_phongo_packedarray_ce)))) {
+			phongo_throw_exception(PHONGO_ERROR_UNEXPECTED_VALUE,
+								   "Expected %s::%s() to return an array, stdClass, %s, or %s, %s given",
+								   ZSTR_VAL(ce->name),
+								   BSON_SERIALIZE_FUNC_NAME,
+								   ZSTR_VAL(php_phongo_document_ce->name),
+								   ZSTR_VAL(php_phongo_packedarray_ce->name),
+								   zend_zval_type_name(retval));
+			return false;
+		}
+
+		return true;
+	}
+
+	phongo_throw_exception(PHONGO_ERROR_UNEXPECTED_VALUE,
+						   "Expected to receive instance of %s, %s given",
+						   ZSTR_VAL(php_phongo_serializable_ce->name),
+						   ZSTR_VAL(ce->name));
+	return false;
+}
+
 /* Appends the array or object argument to the BSON document.
  *
  * For instances of MongoDB\BSON\Document, raw BSON data is appended as document.
@@ -103,24 +152,26 @@ static void php_phongo_bson_append_object(bson_t* bson, php_phongo_field_path* f
 		return;
 	}
 
-	if (Z_TYPE_P(object) == IS_OBJECT && (instanceof_function(Z_OBJCE_P(object), php_phongo_document_ce) || instanceof_function(Z_OBJCE_P(object), php_phongo_packedarray_ce))) {
+	if (Z_TYPE_P(object) == IS_OBJECT && instanceof_function(Z_OBJCE_P(object), php_phongo_type_ce)) {
 		if (instanceof_function(Z_OBJCE_P(object), php_phongo_document_ce)) {
 			php_phongo_document_t* intern = Z_DOCUMENT_OBJ_P(object);
 			bson_append_document(bson, key, key_len, intern->bson);
-		} else {
-			php_phongo_packedarray_t* intern = Z_PACKEDARRAY_OBJ_P(object);
-			bson_append_array(bson, key, key_len, intern->bson);
+
+			return;
 		}
 
-		return;
-	}
+		if (instanceof_function(Z_OBJCE_P(object), php_phongo_packedarray_ce)) {
+			php_phongo_packedarray_t* intern = Z_PACKEDARRAY_OBJ_P(object);
+			bson_append_array(bson, key, key_len, intern->bson);
+
+			return;
+		}
 
-	if (Z_TYPE_P(object) == IS_OBJECT && instanceof_function(Z_OBJCE_P(object), php_phongo_type_ce)) {
 		if (instanceof_function(Z_OBJCE_P(object), php_phongo_serializable_ce)) {
 			zval   obj_data;
 			bson_t child;
 
-			zend_call_method_with_0_params(PHONGO_COMPAT_OBJ_P(object), NULL, NULL, BSON_SERIALIZE_FUNC_NAME, &obj_data);
+			zend_call_method_with_0_params(Z_OBJ_P(object), NULL, NULL, BSON_SERIALIZE_FUNC_NAME, &obj_data);
 
 			if (Z_ISUNDEF(obj_data)) {
 				/* zend_call_method() failed or bsonSerialize() threw an
@@ -128,20 +179,15 @@ static void php_phongo_bson_append_object(bson_t* bson, php_phongo_field_path* f
 				return;
 			}
 
-			if (Z_TYPE(obj_data) != IS_ARRAY && !(Z_TYPE(obj_data) == IS_OBJECT && instanceof_function(Z_OBJCE(obj_data), zend_standard_class_def))) {
-				phongo_throw_exception(PHONGO_ERROR_UNEXPECTED_VALUE,
-									   "Expected %s::%s() to return an array or stdClass, %s given",
-									   ZSTR_VAL(Z_OBJCE_P(object)->name),
-									   BSON_SERIALIZE_FUNC_NAME,
-									   PHONGO_ZVAL_CLASS_OR_TYPE_NAME(obj_data));
+			if (!phongo_check_bson_serialize_return_type(&obj_data, Z_OBJCE_P(object))) {
+				// Exception already thrown
 				zval_ptr_dtor(&obj_data);
-
 				return;
 			}
 
 			/* Persistable objects must always be serialized as BSON documents;
 			 * otherwise, infer based on bsonSerialize()'s return value. */
-			if (instanceof_function(Z_OBJCE_P(object), php_phongo_persistable_ce) || php_phongo_is_array_or_document(&obj_data) == IS_OBJECT) {
+			if (instanceof_function(Z_OBJCE_P(object), php_phongo_persistable_ce) || php_phongo_is_array_or_document(&obj_data) != IS_ARRAY) {
 				bson_append_document_begin(bson, key, key_len, &child);
 				if (instanceof_function(Z_OBJCE_P(object), php_phongo_persistable_ce)) {
 					bson_append_binary(&child, PHONGO_ODM_FIELD_NAME, -1, 0x80, (const uint8_t*) Z_OBJCE_P(object)->name->val, Z_OBJCE_P(object)->name->len);
@@ -162,7 +208,6 @@ static void php_phongo_bson_append_object(bson_t* bson, php_phongo_field_path* f
 			bson_oid_t             oid;
 			php_phongo_objectid_t* intern = Z_OBJECTID_OBJ_P(object);
 
-			mongoc_log(MONGOC_LOG_LEVEL_TRACE, MONGOC_LOG_DOMAIN, "encoding ObjectId");
 			bson_oid_init_from_string(&oid, intern->oid);
 			bson_append_oid(bson, key, key_len, &oid);
 			return;
@@ -170,35 +215,30 @@ static void php_phongo_bson_append_object(bson_t* bson, php_phongo_field_path* f
 		if (instanceof_function(Z_OBJCE_P(object), php_phongo_utcdatetime_ce)) {
 			php_phongo_utcdatetime_t* intern = Z_UTCDATETIME_OBJ_P(object);
 
-			mongoc_log(MONGOC_LOG_LEVEL_TRACE, MONGOC_LOG_DOMAIN, "encoding UTCDateTime");
 			bson_append_date_time(bson, key, key_len, intern->milliseconds);
 			return;
 		}
 		if (instanceof_function(Z_OBJCE_P(object), php_phongo_binary_ce)) {
 			php_phongo_binary_t* intern = Z_BINARY_OBJ_P(object);
 
-			mongoc_log(MONGOC_LOG_LEVEL_TRACE, MONGOC_LOG_DOMAIN, "encoding Binary");
 			bson_append_binary(bson, key, key_len, intern->type, (const uint8_t*) intern->data, (uint32_t) intern->data_len);
 			return;
 		}
 		if (instanceof_function(Z_OBJCE_P(object), php_phongo_decimal128_ce)) {
 			php_phongo_decimal128_t* intern = Z_DECIMAL128_OBJ_P(object);
 
-			mongoc_log(MONGOC_LOG_LEVEL_TRACE, MONGOC_LOG_DOMAIN, "encoding Decimal128");
 			bson_append_decimal128(bson, key, key_len, &intern->decimal);
 			return;
 		}
 		if (instanceof_function(Z_OBJCE_P(object), php_phongo_int64_ce)) {
 			php_phongo_int64_t* intern = Z_INT64_OBJ_P(object);
 
-			mongoc_log(MONGOC_LOG_LEVEL_TRACE, MONGOC_LOG_DOMAIN, "encoding Int64");
 			bson_append_int64(bson, key, key_len, intern->integer);
 			return;
 		}
 		if (instanceof_function(Z_OBJCE_P(object), php_phongo_regex_ce)) {
 			php_phongo_regex_t* intern = Z_REGEX_OBJ_P(object);
 
-			mongoc_log(MONGOC_LOG_LEVEL_TRACE, MONGOC_LOG_DOMAIN, "encoding Regex");
 			bson_append_regex(bson, key, key_len, intern->pattern, intern->flags);
 			return;
 		}
@@ -206,10 +246,8 @@ static void php_phongo_bson_append_object(bson_t* bson, php_phongo_field_path* f
 			php_phongo_javascript_t* intern = Z_JAVASCRIPT_OBJ_P(object);
 
 			if (intern->scope) {
-				mongoc_log(MONGOC_LOG_LEVEL_TRACE, MONGOC_LOG_DOMAIN, "encoding Javascript with scope");
 				bson_append_code_with_scope(bson, key, key_len, intern->code, intern->scope);
 			} else {
-				mongoc_log(MONGOC_LOG_LEVEL_TRACE, MONGOC_LOG_DOMAIN, "encoding Javascript without scope");
 				bson_append_code(bson, key, key_len, intern->code);
 			}
 			return;
@@ -217,17 +255,14 @@ static void php_phongo_bson_append_object(bson_t* bson, php_phongo_field_path* f
 		if (instanceof_function(Z_OBJCE_P(object), php_phongo_timestamp_ce)) {
 			php_phongo_timestamp_t* intern = Z_TIMESTAMP_OBJ_P(object);
 
-			mongoc_log(MONGOC_LOG_LEVEL_TRACE, MONGOC_LOG_DOMAIN, "encoding Timestamp");
 			bson_append_timestamp(bson, key, key_len, intern->timestamp, intern->increment);
 			return;
 		}
 		if (instanceof_function(Z_OBJCE_P(object), php_phongo_maxkey_ce)) {
-			mongoc_log(MONGOC_LOG_LEVEL_TRACE, MONGOC_LOG_DOMAIN, "encoding MaxKey");
 			bson_append_maxkey(bson, key, key_len);
 			return;
 		}
 		if (instanceof_function(Z_OBJCE_P(object), php_phongo_minkey_ce)) {
-			mongoc_log(MONGOC_LOG_LEVEL_TRACE, MONGOC_LOG_DOMAIN, "encoding MinKey");
 			bson_append_minkey(bson, key, key_len);
 			return;
 		}
@@ -237,7 +272,6 @@ static void php_phongo_bson_append_object(bson_t* bson, php_phongo_field_path* f
 			bson_oid_t              oid;
 			php_phongo_dbpointer_t* intern = Z_DBPOINTER_OBJ_P(object);
 
-			mongoc_log(MONGOC_LOG_LEVEL_TRACE, MONGOC_LOG_DOMAIN, "encoding DBPointer");
 			bson_oid_init_from_string(&oid, intern->id);
 			bson_append_dbpointer(bson, key, key_len, intern->ref, &oid);
 			return;
@@ -245,12 +279,10 @@ static void php_phongo_bson_append_object(bson_t* bson, php_phongo_field_path* f
 		if (instanceof_function(Z_OBJCE_P(object), php_phongo_symbol_ce)) {
 			php_phongo_symbol_t* intern = Z_SYMBOL_OBJ_P(object);
 
-			mongoc_log(MONGOC_LOG_LEVEL_TRACE, MONGOC_LOG_DOMAIN, "encoding Symbol");
 			bson_append_symbol(bson, key, key_len, intern->symbol, intern->symbol_len);
 			return;
 		}
 		if (instanceof_function(Z_OBJCE_P(object), php_phongo_undefined_ce)) {
-			mongoc_log(MONGOC_LOG_LEVEL_TRACE, MONGOC_LOG_DOMAIN, "encoding Undefined");
 			bson_append_undefined(bson, key, key_len);
 			return;
 		}
@@ -259,7 +291,6 @@ static void php_phongo_bson_append_object(bson_t* bson, php_phongo_field_path* f
 		return;
 	}
 
-#if PHP_VERSION_ID >= 80100
 	if (Z_TYPE_P(object) == IS_OBJECT && Z_OBJCE_P(object)->ce_flags & ZEND_ACC_ENUM) {
 		if (Z_OBJCE_P(object)->enum_backing_type == IS_UNDEF) {
 			char* path_string = php_phongo_field_path_as_string(field_path);
@@ -271,12 +302,10 @@ static void php_phongo_bson_append_object(bson_t* bson, php_phongo_field_path* f
 		php_phongo_bson_append(bson, field_path, flags, key, key_len, zend_enum_fetch_case_value(Z_OBJ_P(object)));
 		return;
 	}
-#endif /* PHP_VERSION_ID >= 80100 */
 
 	{
 		bson_t child;
 
-		mongoc_log(MONGOC_LOG_LEVEL_TRACE, MONGOC_LOG_DOMAIN, "encoding document");
 		bson_append_document_begin(bson, key, key_len, &child);
 		php_phongo_zval_to_bson_internal(object, field_path, flags, &child, NULL);
 		bson_append_document_end(bson, &child);
@@ -333,11 +362,15 @@ try_again:
 					break;
 				}
 
+				if (!php_phongo_field_path_push(field_path, NULL, PHONGO_FIELD_PATH_ITEM_ARRAY)) {
+					phongo_throw_exception(PHONGO_ERROR_UNEXPECTED_VALUE, "Nesting level too deep");
+					php_phongo_zend_hash_apply_protection_end(tmp_ht);
+					break;
+				}
+
 				bson_append_array_begin(bson, key, key_len, &child);
-				php_phongo_field_path_write_type_at_current_level(field_path, PHONGO_FIELD_PATH_ITEM_ARRAY);
-				field_path->size++;
 				php_phongo_zval_to_bson_internal(entry, field_path, flags, &child, NULL);
-				field_path->size--;
+				php_phongo_field_path_pop(field_path);
 				bson_append_array_end(bson, &child);
 
 				php_phongo_zend_hash_apply_protection_end(tmp_ht);
@@ -356,14 +389,21 @@ try_again:
 			}
 
 			if (Z_TYPE_P(entry) == IS_OBJECT && instanceof_function(Z_OBJCE_P(entry), php_phongo_packedarray_ce)) {
-				php_phongo_field_path_write_type_at_current_level(field_path, PHONGO_FIELD_PATH_ITEM_ARRAY);
+				if (!php_phongo_field_path_push(field_path, NULL, PHONGO_FIELD_PATH_ITEM_ARRAY)) {
+					phongo_throw_exception(PHONGO_ERROR_UNEXPECTED_VALUE, "Nesting level too deep");
+					php_phongo_zend_hash_apply_protection_end(tmp_ht);
+					break;
+				}
 			} else {
-				php_phongo_field_path_write_type_at_current_level(field_path, PHONGO_FIELD_PATH_ITEM_DOCUMENT);
+				if (!php_phongo_field_path_push(field_path, NULL, PHONGO_FIELD_PATH_ITEM_DOCUMENT)) {
+					phongo_throw_exception(PHONGO_ERROR_UNEXPECTED_VALUE, "Nesting level too deep");
+					php_phongo_zend_hash_apply_protection_end(tmp_ht);
+					break;
+				}
 			}
 
-			field_path->size++;
 			php_phongo_bson_append_object(bson, field_path, flags, key, key_len, entry);
-			field_path->size--;
+			php_phongo_field_path_pop(field_path);
 
 			php_phongo_zend_hash_apply_protection_end(tmp_ht);
 			break;
@@ -433,6 +473,15 @@ static void php_phongo_zval_to_bson_internal(zval* data, php_phongo_field_path*
 			}
 
 			if (instanceof_function(Z_OBJCE_P(data), php_phongo_packedarray_ce)) {
+				/* If we are at the root-level, PackedArray instances should be
+				 * prohibited unless PHONGO_BSON_ALLOW_ROOT_ARRAY is set. */
+				bool is_root_level = (field_path->size == 0);
+
+				if (is_root_level && !(flags & PHONGO_BSON_ALLOW_ROOT_ARRAY)) {
+					phongo_throw_exception(PHONGO_ERROR_UNEXPECTED_VALUE, "%s cannot be serialized as a root document", ZSTR_VAL(Z_OBJCE_P(data)->name));
+					return;
+				}
+
 				php_phongo_packedarray_t* intern = Z_PACKEDARRAY_OBJ_P(data);
 
 				phongo_bson_copy_to_noinit(intern->bson, bson);
@@ -443,7 +492,7 @@ static void php_phongo_zval_to_bson_internal(zval* data, php_phongo_field_path*
 			/* For any MongoDB\BSON\Serializable, invoke the bsonSerialize method
 			 * and work with the result. */
 			if (instanceof_function(Z_OBJCE_P(data), php_phongo_serializable_ce)) {
-				zend_call_method_with_0_params(PHONGO_COMPAT_OBJ_P(data), NULL, NULL, BSON_SERIALIZE_FUNC_NAME, &obj_data);
+				zend_call_method_with_0_params(Z_OBJ_P(data), NULL, NULL, BSON_SERIALIZE_FUNC_NAME, &obj_data);
 
 				if (Z_ISUNDEF(obj_data)) {
 					/* zend_call_method() failed or bsonSerialize() threw an
@@ -451,19 +500,11 @@ static void php_phongo_zval_to_bson_internal(zval* data, php_phongo_field_path*
 					return;
 				}
 
-				if (Z_TYPE(obj_data) != IS_ARRAY && !(Z_TYPE(obj_data) == IS_OBJECT && instanceof_function(Z_OBJCE(obj_data), zend_standard_class_def))) {
-					phongo_throw_exception(
-						PHONGO_ERROR_UNEXPECTED_VALUE,
-						"Expected %s::%s() to return an array or stdClass, %s given",
-						ZSTR_VAL(Z_OBJCE_P(data)->name),
-						BSON_SERIALIZE_FUNC_NAME,
-						PHONGO_ZVAL_CLASS_OR_TYPE_NAME(obj_data));
-
+				if (!phongo_check_bson_serialize_return_type(&obj_data, Z_OBJCE_P(data))) {
+					// Exception already thrown
 					goto cleanup;
 				}
 
-				ht_data = HASH_OF(&obj_data);
-
 				if (instanceof_function(Z_OBJCE_P(data), php_phongo_persistable_ce)) {
 					bson_append_binary(bson, PHONGO_ODM_FIELD_NAME, -1, 0x80, (const uint8_t*) Z_OBJCE_P(data)->name->val, Z_OBJCE_P(data)->name->len);
 					/* Ensure that we ignore an existing key with the same name
@@ -471,25 +512,32 @@ static void php_phongo_zval_to_bson_internal(zval* data, php_phongo_field_path*
 					skip_odm_field = true;
 				}
 
+				// If bsonSerialize() returns a BSON document or packedArray instance, recurse to copy data over directly
+				if (Z_TYPE(obj_data) == IS_OBJECT && (instanceof_function(Z_OBJCE(obj_data), php_phongo_document_ce) || instanceof_function(Z_OBJCE(obj_data), php_phongo_packedarray_ce))) {
+					php_phongo_zval_to_bson_internal(&obj_data, field_path, flags, bson, bson_out);
+
+					goto done;
+				}
+
+				ht_data = HASH_OF(&obj_data);
+
 				break;
 			}
 
 			/* For the error handling that follows, we can safely assume that we
 			 * are at the root level, since php_phongo_bson_append_object would
 			 * have already been called for a non-root level. */
-#if PHP_VERSION_ID >= 80100
 			if (Z_OBJCE_P(data)->ce_flags & ZEND_ACC_ENUM) {
 				phongo_throw_exception(PHONGO_ERROR_UNEXPECTED_VALUE, "Enum %s cannot be serialized as a root element", ZSTR_VAL(Z_OBJCE_P(data)->name));
 				return;
 			}
-#endif /* PHP_VERSION_ID >= 80100 */
 
 			if (instanceof_function(Z_OBJCE_P(data), php_phongo_type_ce)) {
 				phongo_throw_exception(PHONGO_ERROR_UNEXPECTED_VALUE, "%s instance %s cannot be serialized as a root element", ZSTR_VAL(php_phongo_type_ce->name), ZSTR_VAL(Z_OBJCE_P(data)->name));
 				return;
 			}
 
-			ht_data                 = Z_OBJ_HT_P(data)->get_properties(PHONGO_COMPAT_OBJ_P(data));
+			ht_data                 = Z_OBJ_HT_P(data)->get_properties(Z_OBJ_P(data));
 			ht_data_from_properties = true;
 			break;
 
@@ -553,7 +601,6 @@ done:
 
 		bson_oid_init(&oid, NULL);
 		bson_append_oid(bson, "_id", strlen("_id"), &oid);
-		mongoc_log(MONGOC_LOG_LEVEL_TRACE, MONGOC_LOG_DOMAIN, "Added new _id");
 	}
 
 	if (flags & PHONGO_BSON_RETURN_ID && bson_out) {
