@@ -1001,10 +1001,10 @@ static bool php_phongo_extract_handshake_data(zval* driver, const char* key, cha
 		return true;
 	}
 
-	zvalue = php_array_fetch(driver, key);
+	zvalue = php_array_fetch_deref(driver, key);
 
 	if (Z_TYPE_P(zvalue) != IS_STRING) {
-		phongo_throw_exception(PHONGO_ERROR_INVALID_ARGUMENT, "Expected \"%s\" handshake option to be a string, %s given", key, PHONGO_ZVAL_CLASS_OR_TYPE_NAME_P(zvalue));
+		phongo_throw_exception(PHONGO_ERROR_INVALID_ARGUMENT, "Expected \"%s\" handshake option to be a string, %s given", key, zend_zval_type_name(zvalue));
 		return false;
 	}
 
@@ -1076,10 +1076,10 @@ static void php_phongo_set_handshake_data(zval* driverOptions)
 	size_t platform_len = 0;
 
 	if (driverOptions && php_array_existsc(driverOptions, "driver")) {
-		zval* driver = php_array_fetchc(driverOptions, "driver");
+		zval* driver = php_array_fetchc_deref(driverOptions, "driver");
 
 		if (Z_TYPE_P(driver) != IS_ARRAY) {
-			phongo_throw_exception(PHONGO_ERROR_INVALID_ARGUMENT, "Expected \"driver\" driver option to be an array, %s given", PHONGO_ZVAL_CLASS_OR_TYPE_NAME_P(driver));
+			phongo_throw_exception(PHONGO_ERROR_INVALID_ARGUMENT, "Expected \"driver\" driver option to be an array, %s given", zend_zval_type_name(driver));
 			return;
 		}
 
@@ -1128,7 +1128,7 @@ static mongoc_client_t* php_phongo_make_mongo_client(const mongoc_uri_t* uri, zv
 #ifdef HAVE_SYSTEM_LIBBSON
 	bson_version = bson_get_version();
 #else
-	bson_version   = "bundled";
+	bson_version = "bundled";
 #endif
 
 	MONGOC_DEBUG(
@@ -1229,10 +1229,10 @@ static bool phongo_manager_set_serverapi_opts(php_phongo_manager_t* manager, zva
 		return true;
 	}
 
-	zServerApi = php_array_fetch(driverOptions, "serverApi");
+	zServerApi = php_array_fetchc_deref(driverOptions, "serverApi");
 
 	if (Z_TYPE_P(zServerApi) != IS_OBJECT || !instanceof_function(Z_OBJCE_P(zServerApi), php_phongo_serverapi_ce)) {
-		phongo_throw_exception(PHONGO_ERROR_INVALID_ARGUMENT, "Expected \"serverApi\" driver option to be %s, %s given", ZSTR_VAL(php_phongo_serverapi_ce->name), PHONGO_ZVAL_CLASS_OR_TYPE_NAME_P(zServerApi));
+		phongo_throw_exception(PHONGO_ERROR_INVALID_ARGUMENT, "Expected \"serverApi\" driver option to be %s, %s given", ZSTR_VAL(php_phongo_serverapi_ce->name), zend_zval_type_name(zServerApi));
 		return false;
 	}
 
@@ -1258,29 +1258,29 @@ static bool phongo_manager_set_auto_encryption_opts(php_phongo_manager_t* manage
 		return true;
 	}
 
-	zAutoEncryptionOpts = php_array_fetch(driverOptions, "autoEncryption");
+	zAutoEncryptionOpts = php_array_fetchc_deref(driverOptions, "autoEncryption");
 
 	if (Z_TYPE_P(zAutoEncryptionOpts) != IS_ARRAY) {
-		phongo_throw_exception(PHONGO_ERROR_INVALID_ARGUMENT, "Expected \"autoEncryption\" driver option to be array, %s given", PHONGO_ZVAL_CLASS_OR_TYPE_NAME_P(zAutoEncryptionOpts));
+		phongo_throw_exception(PHONGO_ERROR_INVALID_ARGUMENT, "Expected \"autoEncryption\" driver option to be array, %s given", zend_zval_type_name(zAutoEncryptionOpts));
 		return false;
 	}
 
 	auto_encryption_opts = mongoc_auto_encryption_opts_new();
 
 	if (php_array_existsc(zAutoEncryptionOpts, "bypassAutoEncryption")) {
-		mongoc_auto_encryption_opts_set_bypass_auto_encryption(auto_encryption_opts, php_array_fetch_bool(zAutoEncryptionOpts, "bypassAutoEncryption"));
+		mongoc_auto_encryption_opts_set_bypass_auto_encryption(auto_encryption_opts, php_array_fetchc_bool(zAutoEncryptionOpts, "bypassAutoEncryption"));
 	}
 
 	if (php_array_existsc(zAutoEncryptionOpts, "bypassQueryAnalysis")) {
-		mongoc_auto_encryption_opts_set_bypass_query_analysis(auto_encryption_opts, php_array_fetch_bool(zAutoEncryptionOpts, "bypassQueryAnalysis"));
+		mongoc_auto_encryption_opts_set_bypass_query_analysis(auto_encryption_opts, php_array_fetchc_bool(zAutoEncryptionOpts, "bypassQueryAnalysis"));
 	}
 
 	if (php_array_existsc(zAutoEncryptionOpts, "encryptedFieldsMap")) {
-		zval*  enc_fields_map = php_array_fetch(zAutoEncryptionOpts, "encryptedFieldsMap");
+		zval*  enc_fields_map = php_array_fetchc_deref(zAutoEncryptionOpts, "encryptedFieldsMap");
 		bson_t bson_map       = BSON_INITIALIZER;
 
 		if (Z_TYPE_P(enc_fields_map) != IS_OBJECT && Z_TYPE_P(enc_fields_map) != IS_ARRAY) {
-			phongo_throw_exception(PHONGO_ERROR_INVALID_ARGUMENT, "Expected \"encryptedFieldsMap\" autoEncryption option to be an array or object, %s given", PHONGO_ZVAL_CLASS_OR_TYPE_NAME_P(enc_fields_map));
+			phongo_throw_exception(PHONGO_ERROR_INVALID_ARGUMENT, "Expected \"encryptedFieldsMap\" autoEncryption option to be an array or object, %s given", zend_zval_type_name(enc_fields_map));
 			goto cleanup;
 		}
 
@@ -1299,10 +1299,10 @@ static bool phongo_manager_set_auto_encryption_opts(php_phongo_manager_t* manage
 	}
 
 	if (php_array_existsc(zAutoEncryptionOpts, "keyVaultClient")) {
-		zval* key_vault_client = php_array_fetch(zAutoEncryptionOpts, "keyVaultClient");
+		zval* key_vault_client = php_array_fetchc_deref(zAutoEncryptionOpts, "keyVaultClient");
 
 		if (Z_TYPE_P(key_vault_client) != IS_OBJECT || !instanceof_function(Z_OBJCE_P(key_vault_client), php_phongo_manager_ce)) {
-			phongo_throw_exception(PHONGO_ERROR_INVALID_ARGUMENT, "Expected \"keyVaultClient\" autoEncryption option to be %s, %s given", ZSTR_VAL(php_phongo_manager_ce->name), PHONGO_ZVAL_CLASS_OR_TYPE_NAME_P(key_vault_client));
+			phongo_throw_exception(PHONGO_ERROR_INVALID_ARGUMENT, "Expected \"keyVaultClient\" autoEncryption option to be %s, %s given", ZSTR_VAL(php_phongo_manager_ce->name), zend_zval_type_name(key_vault_client));
 			goto cleanup;
 		}
 
@@ -1329,7 +1329,7 @@ static bool phongo_manager_set_auto_encryption_opts(php_phongo_manager_t* manage
 		int       plen;
 		zend_bool pfree;
 
-		key_vault_ns = php_array_fetch_string(zAutoEncryptionOpts, "keyVaultNamespace", &plen, &pfree);
+		key_vault_ns = php_array_fetchc_string(zAutoEncryptionOpts, "keyVaultNamespace", &plen, &pfree);
 
 		if (!phongo_split_namespace(key_vault_ns, &db_name, &coll_name)) {
 			phongo_throw_exception(PHONGO_ERROR_INVALID_ARGUMENT, "Expected \"keyVaultNamespace\" autoEncryption option to contain a full collection namespace");
@@ -1352,11 +1352,11 @@ static bool phongo_manager_set_auto_encryption_opts(php_phongo_manager_t* manage
 	}
 
 	if (php_array_existsc(zAutoEncryptionOpts, "kmsProviders")) {
-		zval*  kms_providers  = php_array_fetch(zAutoEncryptionOpts, "kmsProviders");
+		zval*  kms_providers  = php_array_fetchc_deref(zAutoEncryptionOpts, "kmsProviders");
 		bson_t bson_providers = BSON_INITIALIZER;
 
 		if (Z_TYPE_P(kms_providers) != IS_OBJECT && Z_TYPE_P(kms_providers) != IS_ARRAY) {
-			phongo_throw_exception(PHONGO_ERROR_INVALID_ARGUMENT, "Expected \"kmsProviders\" autoEncryption option to be an array or object, %s given", PHONGO_ZVAL_CLASS_OR_TYPE_NAME_P(kms_providers));
+			phongo_throw_exception(PHONGO_ERROR_INVALID_ARGUMENT, "Expected \"kmsProviders\" autoEncryption option to be an array or object, %s given", zend_zval_type_name(kms_providers));
 			goto cleanup;
 		}
 
@@ -1371,11 +1371,11 @@ static bool phongo_manager_set_auto_encryption_opts(php_phongo_manager_t* manage
 	}
 
 	if (php_array_existsc(zAutoEncryptionOpts, "schemaMap")) {
-		zval*  schema_map = php_array_fetch(zAutoEncryptionOpts, "schemaMap");
+		zval*  schema_map = php_array_fetchc_deref(zAutoEncryptionOpts, "schemaMap");
 		bson_t bson_map   = BSON_INITIALIZER;
 
 		if (Z_TYPE_P(schema_map) != IS_OBJECT && Z_TYPE_P(schema_map) != IS_ARRAY) {
-			phongo_throw_exception(PHONGO_ERROR_INVALID_ARGUMENT, "Expected \"schemaMap\" autoEncryption option to be an array or object, %s given", PHONGO_ZVAL_CLASS_OR_TYPE_NAME_P(schema_map));
+			phongo_throw_exception(PHONGO_ERROR_INVALID_ARGUMENT, "Expected \"schemaMap\" autoEncryption option to be an array or object, %s given", zend_zval_type_name(schema_map));
 			goto cleanup;
 		}
 
@@ -1390,11 +1390,11 @@ static bool phongo_manager_set_auto_encryption_opts(php_phongo_manager_t* manage
 	}
 
 	if (php_array_existsc(zAutoEncryptionOpts, "tlsOptions")) {
-		zval*  tls_options  = php_array_fetch(zAutoEncryptionOpts, "tlsOptions");
+		zval*  tls_options  = php_array_fetchc_deref(zAutoEncryptionOpts, "tlsOptions");
 		bson_t bson_options = BSON_INITIALIZER;
 
 		if (Z_TYPE_P(tls_options) != IS_OBJECT && Z_TYPE_P(tls_options) != IS_ARRAY) {
-			phongo_throw_exception(PHONGO_ERROR_INVALID_ARGUMENT, "Expected \"tlsOptions\" autoEncryption option to be an array or object, %s given", PHONGO_ZVAL_CLASS_OR_TYPE_NAME_P(tls_options));
+			phongo_throw_exception(PHONGO_ERROR_INVALID_ARGUMENT, "Expected \"tlsOptions\" autoEncryption option to be an array or object, %s given", zend_zval_type_name(tls_options));
 			goto cleanup;
 		}
 
@@ -1409,11 +1409,11 @@ static bool phongo_manager_set_auto_encryption_opts(php_phongo_manager_t* manage
 	}
 
 	if (php_array_existsc(zAutoEncryptionOpts, "extraOptions")) {
-		zval*  extra_options = php_array_fetch(zAutoEncryptionOpts, "extraOptions");
+		zval*  extra_options = php_array_fetchc_deref(zAutoEncryptionOpts, "extraOptions");
 		bson_t bson_options  = BSON_INITIALIZER;
 
 		if (Z_TYPE_P(extra_options) != IS_OBJECT && Z_TYPE_P(extra_options) != IS_ARRAY) {
-			phongo_throw_exception(PHONGO_ERROR_INVALID_ARGUMENT, "Expected \"extraOptions\" autoEncryption option to be an array or object, %s given", PHONGO_ZVAL_CLASS_OR_TYPE_NAME_P(extra_options));
+			phongo_throw_exception(PHONGO_ERROR_INVALID_ARGUMENT, "Expected \"extraOptions\" autoEncryption option to be an array or object, %s given", zend_zval_type_name(extra_options));
 			goto cleanup;
 		}
 
